----------------------------------------------------------------------
-- 6.1 Gear box model 
--
-- Copyright (C) 2017 JSAE
-- ----------------------------------------------------------------------------
-- Copyright (C) 2004-2007 VDA/FAT
--
-- This model is a component of the open source library created by the VDA/FAT 
-- working group number 30 and is covered by this license agreement. This model
-- including any updates, modifications, revisions, copies, and documentation
-- are copyrighted works of the VDA/FAT. Use of this model indicates your
-- complete and unconditional acceptance of the terms and conditions set forth
-- in this license agreement.
--  
-- The VDA/FAT grants a non-exclusive license to use, reproduce, modify and
-- distribute this model under the condition, that:
-- (a) no fee or other consideration is charged for any distribution except 
--     compilations distributed in accordance with Section (d) of this license 
--     agreement;
-- (b) the comment text embedded in this model is included verbatim in each
--     copy of this model made or distributed by you, whether or not such
--     version is modified;
-- (c) any modified version must include a conspicuous notice that this model
--     has been modified and the date of modification; and
-- (d) any compilations sold by you that include this model must include a 
--     conspicuous notice that this model is available from the VDA/FAT in 
--     its original form at no charge.
--
-- THIS MODEL IS LICENSED TO YOU "AS IT IS" AND WITH NO WARRANTIES, EXPRESSED
-- OR IMPLIED. THE VDA/FAT AND ALL COMPANIES CONTRIBUTING TO THIS LIBRARY
-- SPECIFICALLY DISCLAIM ALL IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
-- FOR A PARTICULAR PURPOSE. THE VDA/FAT AND ALL COMPANIES ORGANIZED WITHIN
-- MUST NOT HAVE ANY RESPONSIBILITY FOR ANY DAMAGES, FINANCIAL OR LEGAL CLAIMS 
-- WHATEVER.
-- ----------------------------------------------------------------------------
-- Source:
-- gearbox.vhd
--
-- Development simulator:
--  
--
-- Tested on: 
-- ------------------------------------------------------------------
-- Version  Simulator      Version   OS    Validator      Date
-- ------------------------------------------------------------------
-- 1.1      
--
-- Modification History:
-- ------------------------------------------------------------------
-- Version  Author            Date           Description
-- ------------------------------------------------------------------
--  1.1    Joachim Haase      2007/03/26     Original Version     
-- 
--
-- ----------------------------------------------------------------------------
-- $Version: 2.0.0 $
-- $Revision: 1746 $
-- $Date: 2007-01-05 16:38:40 +0100 (Fri, 05 Jan 2007) $
-- $Author: haase $
-- ----------------------------------------------------------------------------
	

--/**
--	
-- Gearbox.
-- The model describes a simple gearbox for hybrid electric
-- drive trains. 
-- 
-- @generic   RATIO      gear relationship (RATIO(I)=gear ratio for gear i)
-- @generic   ETA        efficiency
-- @generic   TC         rise and fall times to switch torque
-- 
-- @port      S_GEAR     gear
-- @port      MRV_IN	 torque input
-- @port      MRV_OUT    torque output
--	
-- @test      bench


-- @reference Ehsani, M.; Gao, Y.; Gay, S. E.; Emadi, A.:
--            Modern Electric, Hybrid Electric, and Fuel Cell Vehicles.
--            CRC Press, 2005.
--
--*/					
----------------------------------------------------------------------
library IEEE ; 
    use IEEE.MECHANICAL_SYSTEMS.all;
    use IEEE.MATH_REAL.all;
----------------------------------------------------------------------
entity GEARBOX is
  generic (
    TC    : REAL := 0.1 -- rise/fall times
  );
  port    (
    quantity Q_in_gear :in REAL ;
    terminal MRV_IN  : ROTATIONAL_VELOCITY;  -- provides torque
    terminal MRV_OUT : ROTATIONAL_VELOCITY   -- output torque
  );
end entity GEARBOX;
----------------------------------------------------------------------
architecture A0 of  GEARBOX is
  constant RATIO : REAL_VECTOR := (0.0, 3.250, 1.782, 1.172, 0.909, 0.702);    -- gear ratios
  constant ETA   : REAL_VECTOR := (0.96, 0.96, 0.96, 0.96, 0.96, 0.96);    -- efficiency

  quantity ANGULAR_VELOCITY_IN  across TORQUE_IN  through MRV_IN;
  quantity ANGULAR_VELOCITY_OUT across                    MRV_OUT;
  quantity                             TORQUE_OUT through ROTATIONAL_VELOCITY_REF to MRV_OUT;
    
  constant sample_time : TIME := 0.1 * 1 sec;
  signal   S_GEAR  : NATURAL;              -- gear position 

  signal RATIO_GEAR        : REAL :=  0.0;
  signal ETA_GEAR        : REAL :=  0.0;
begin

  process
  begin
    s_gear <= integer(Q_in_gear);
    wait for sample_time;
  end process ;

  RATIO_GEAR <= RATIO(S_GEAR);
  ETA_GEAR <= ETA(S_GEAR);  


  ANGULAR_VELOCITY_IN  == RATIO_GEAR'RAMP(TC)*ANGULAR_VELOCITY_OUT;

  if TORQUE_OUT > -0.1 use
    TORQUE_OUT           == ETA_GEAR'RAMP(TC)*RATIO_GEAR'RAMP(TC)*TORQUE_IN;
  else
    TORQUE_OUT*ETA_GEAR'RAMP(TC) == RATIO_GEAR'RAMP(TC)*TORQUE_IN;
  end use;

end architecture A0;
