----------------------------------------------------------------------
-- 8. Brake Model :
--
-- Copyright (C) 2017 JSAE
-- ----------------------------------------------------------------------------
-- Copyright (C) 2004-2007 VDA/FAT
--
-- This model is a component of the open source library created by the VDA/FAT 
-- working group number 30 and is covered by this license agreement. This model
-- including any updates, modifications, revisions, copies, and documentation
-- are copyrighted works of the VDA/FAT. Use of this model indicates your
-- complete and unconditional acceptance of the terms and conditions set forth
-- in this license agreement.
--  
-- The VDA/FAT grants a non-exclusive license to use, reproduce, modify and
-- distribute this model under the condition, that:
-- (a) no fee or other consideration is charged for any distribution except 
--     compilations distributed in accordance with Section (d) of this license 
--     agreement;
-- (b) the comment text embedded in this model is included verbatim in each
--     copy of this model made or distributed by you, whether or not such
--     version is modified;
-- (c) any modified version must include a conspicuous notice that this model
--     has been modified and the date of modification; and
-- (d) any compilations sold by you that include this model must include a 
--     conspicuous notice that this model is available from the VDA/FAT in 
--     its original form at no charge.
--
-- THIS MODEL IS LICENSED TO YOU "AS IT IS" AND WITH NO WARRANTIES, EXPRESSED
-- OR IMPLIED. THE VDA/FAT AND ALL COMPANIES CONTRIBUTING TO THIS LIBRARY
-- SPECIFICALLY DISCLAIM ALL IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
-- FOR A PARTICULAR PURPOSE. THE VDA/FAT AND ALL COMPANIES ORGANIZED WITHIN
-- MUST NOT HAVE ANY RESPONSIBILITY FOR ANY DAMAGES, FINANCIAL OR LEGAL CLAIMS 
-- WHATEVER.
-- ----------------------------------------------------------------------------
-- Source:
-- brake.vhd
--
-- Development simulator:
--  
--
-- Tested on: 
-- ------------------------------------------------------------------
-- Version  Simulator      Version   OS    Validator      Date
-- ------------------------------------------------------------------
-- 1.1      
--
-- Modification History:
-- ------------------------------------------------------------------
-- Version  Author            Date           Description
-- ------------------------------------------------------------------
--  1.1    Joachim Haase      2007/03/26     Original Version     
--  1.2    J. Haase           2007/09/04     BEARING removed
--
-- ----------------------------------------------------------------------------
-- $Version: 2.0.0 $
-- $Revision: 1746 $
-- $Date: 2007-01-05 16:38:40 +0100 (Fri, 05 Jan 2007) $
-- $Author: haase $
-- ----------------------------------------------------------------------------
	

--/**
--	
-- Modelica Rotational Brake Model.
-- <p>The component models a brake. A frictional torque is
-- acting between the housing and a flange and a controlled
-- normal force that presses the flange to the housing.</p>
-- <p>The model describes a brake nearly in the same way
-- as in Modelica but with ROTATIONAL_VELOCITY terminals.</p> 
-- 
-- @generic   W            array with angular velocities [rad/s]
-- @generic   MUE          array with associated friction coefficients
-- @generic   PEAK         scaling for friction coefficients
-- @generic   CGEO         geometry constant containing friction distrution assumptions
-- @generic   FN_MAX       maximum normal force [N]
-- @generic   T0           delay of control value [s]
-- @generic   W_SMALL      velocity that is handeled as zero velocity  [rad/s]
-- 
-- @port      CTRL         control force
-- @port      FLANGE_B      plus terminal
--	
-- @test      bench
-- @reference Fritzon, P.: Priciples of Object-Oriented Modeling
--            and Simulation with Modelica 2.1. IEEE Press, 2004.
--
--*/					

----------------------------------------------------------------------	
library IEEE, FUNDAMENTALS_VDA;
    use IEEE.MECHANICAL_SYSTEMS.all;
    use IEEE.MATH_REAL.all;
    use FUNDAMENTALS_VDA.TLU_VDA.all;
----------------------------------------------------------------------
entity BRAKE is
  generic  (
     PEAK    : REAL := 1.0;   -- scalling
     CGEO    : REAL := 1.0;   -- Geometry coef.
     FN_MAX  : REAL := 650.0; -- Max Force[N]
     W_SMALL : REAL := 0.1    -- speed threshold [rad/s]
    );
  port     (
    quantity Q_in_CTRL      : in REAL;  -- Braking power [0 to 1]
    terminal MRV_B  : ROTATIONAL_VELOCITY
    );  

end entity BRAKE;
----------------------------------------------------------------------
architecture BASIC of BRAKE is
  constant W       : REAL_VECTOR    := (0.0, 1.0); -- wheel speed arry
  constant MUE     : REAL_VECTOR    := (0.5, 0.5); -- mue arry
  constant FN_SMALL  : REAL := FN_MAX*1.0E-3;
  constant MUE_0     : REAL := LOOKUP_1D(0.0, W, MUE);
  constant PEAK_FRIC : REAL := PEAK;

  quantity W_FRIC  across TAU_FRIC  through MRV_B;

  quantity FN                : REAL;
  quantity FN_NORMALIZED     : REAL;

begin

  FN_NORMALIZED  == REALMAX(0.0, REALMIN(1.0, Q_in_CTRL));
  FN             == FN_MAX*FN_NORMALIZED;
  
  if not W_FRIC'ABOVE(-W_SMALL) or W_FRIC'ABOVE(W_SMALL) use
     ----
     -- Stopping torque
     ----
     TAU_FRIC   ==  PEAK_FRIC*CGEO*LOOKUP_1D(abs(W_FRIC),W,MUE)*FN*SIGN(W_FRIC);
  else
     ----
     -- Range in small speed
     ----    
     if not FN'ABOVE(FN_SMALL) or FN < FN_SMALL use
        TAU_FRIC == 0.0;
     else
        W_FRIC   == 0.0;
     end use;
  end use;
  
end architecture BASIC;
