----------------------------------------------------------------------
-- 10. Vehicle model :
--
-- Copyright (C) 2017 JSAE
-- ----------------------------------------------------------------------------
-- Copyright (C) 2004-2007 VDA/FAT
--
-- This model is a component of the open source library created by the VDA/FAT 
-- working group number 30 and is covered by this license agreement. This model
-- including any updates, modifications, revisions, copies, and documentation
-- are copyrighted works of the VDA/FAT. Use of this model indicates your
-- complete and unconditional acceptance of the terms and conditions set forth
-- in this license agreement.
--  
-- The VDA/FAT grants a non-exclusive license to use, reproduce, modify and
-- distribute this model under the condition, that:
-- (a) no fee or other consideration is charged for any distribution except 
--     compilations distributed in accordance with Section (d) of this license 
--     agreement;
-- (b) the comment text embedded in this model is included verbatim in each
--     copy of this model made or distributed by you, whether or not such
--     version is modified;
-- (c) any modified version must include a conspicuous notice that this model
--     has been modified and the date of modification; and
-- (d) any compilations sold by you that include this model must include a 
--     conspicuous notice that this model is available from the VDA/FAT in 
--     its original form at no charge.
--
-- THIS MODEL IS LICENSED TO YOU "AS IT IS" AND WITH NO WARRANTIES, EXPRESSED
-- OR IMPLIED. THE VDA/FAT AND ALL COMPANIES CONTRIBUTING TO THIS LIBRARY
-- SPECIFICALLY DISCLAIM ALL IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
-- FOR A PARTICULAR PURPOSE. THE VDA/FAT AND ALL COMPANIES ORGANIZED WITHIN
-- MUST NOT HAVE ANY RESPONSIBILITY FOR ANY DAMAGES, FINANCIAL OR LEGAL CLAIMS 
-- WHATEVER.
-- ----------------------------------------------------------------------------
-- Source:
-- vehicle.vhd
--
-- Development simulator:
--  
--
-- Tested on: 
-- ------------------------------------------------------------------
-- Version  Simulator      Version   OS    Validator      Date
-- ------------------------------------------------------------------
-- 1.1      
--
-- Modification History:
-- ------------------------------------------------------------------
-- Version  Author            Date           Description
-- ------------------------------------------------------------------
--  1.1    Joachim Haase      2007/03/26     Original Version     
-- 
--
-- ----------------------------------------------------------------------------
-- $Version: 2.0.0 $
-- $Revision: 1746 $
-- $Date: 2007-01-05 16:38:40 +0100 (Fri, 05 Jan 2007) $
-- $Author: haase $
-- ----------------------------------------------------------------------------
	

--/**
--	
-- 1D Vehicle Model.
-- The model describes a simple vehicle model for hybrid electric
-- drive trains. 
-- 
-- @generic   FR0        rolling resistance coefficient
-- @generic   FR1        rolling resistance coefficient of V
-- @generic   FR4        rolling resistance coefficient of V**2
-- @generic   MUE        frictional coefficient
-- @generic   RHO        density of air [kg/m^3]
-- @generic   TAU        time constant [s]
-- 
-- @port      VW 	     wind (directed against car movement) [m/s]
-- @port      ALPHA	     grading angle [deg]
-- @port      MTV        connected to torque
--	
-- @test      bench
--

-- @reference Ehsani, M.; Gao, Y.; Gay, S. E.; Emadi, A.:
--            Modern Electric, Hybrid Electric, and Fuel Cell Vehicles.
--            CRC Press, 2005.
--
-- @reference Gao, D.W.; Mi, C.; Emadi, A.: Modeling and Simulation of
--            Electric and Hybrid Vehicles. Proc. of the IEEE 
--            vol. 95, No. 4, April 2007, pp. 729-745.
--
-- @reference Isermann, R.: Fahrdynamik-Regelung.
--            Vieweg-Verlag, 2006.
--*/					
----------------------------------------------------------------------
library IEEE ;
    use IEEE.MECHANICAL_SYSTEMS.all;
    use IEEE.FUNDAMENTAL_CONSTANTS.all;
    use IEEE.MATH_REAL.all;
----------------------------------------------------------------------
entity VEHICLE is
  generic (
    FR0   : REAL :=    0.01;  -- rolling resistance coefficient
    FR1   : REAL :=    0.0;   -- rolling resistance coefficient of v 
    FR4   : REAL :=    0.0;   -- rolling resistance coefficient of v**4
    MUE   : REAL :=    0.75;  -- frictional coefficient
    RHO   : REAL :=    1.2;   -- density of air [kg/m^3]
    TAU   : REAL :=    0.016;  -- time constant [s]
    VehicleMass : REAL :=  1500.0;  -- VehicleMaa [kg]
    AF    : REAL := 2.0;     -- Frontal projected area [m^2]
    L     : REAL := 2.0;     -- Length of wheel base [m]
    LA    : REAL := 0.5;     -- Gravity center position from front wheel[m]
    RD    : REAL := 0.25;     -- wheel radius [m]
    HG    : REAL := 0.5;     -- Gravity center height [m]
    CD    : REAL := 0.5     -- coefficient of aerodynamic resistance
  );
  port   (
    quantity Q_in_VW    : REAL := 0.0 ;          -- wind [m/s]
    quantity Q_in_ALPHA : REAL := 0.0 ;          -- grading angle [deg}
    quantity Q_out_Speed : out REAL := 0.0 ; --vehicle speed [m/s]
    terminal MTV   : TRANSLATIONAL_VELOCITY
  );             
end entity VEHICLE;
----------------------------------------------------------------------
architecture FRONT_DRIVEN of VEHICLE is
  --------------
  function F_SMOOTH(V, EPS : REAL) return REAL is
    variable RESULT : REAL;
  begin
    if abs V < EPS then
      RESULT := 1.0 - COS(V/EPS*MATH_PI/2.0);
    else
      RESULT := 1.0;
    end if;
    return RESULT;
  end function F_SMOOTH; 
  -------------

  constant EPS : REAL := 1.0;
  constant G   : REAL := PHYS_GRAVITY;  -- gravity constant [m/s^-2]
     
  -- V in [m/s], F in N
  quantity V  across F_EXT through MTV;
  
  -- Vehicle Mass
  quantity Vmass across Fmass through MTV to TRANSLATIONAL_V_REF ;

  quantity F_ROLLING : REAL;
  quantity F_CAR     : REAL;
  quantity F_DRAG    : REAL; 
  quantity F_GRAD    : REAL;
  quantity F_MAX     : REAL;
     
begin

  IF (domain = quiescent_domain) USE
    -- rolling resistance
    -- normal force times rolling resistance coefficient
    -- (note that V in given in m/s)
    F_ROLLING == VehicleMass*COS(Q_in_ALPHA/180.0*MATH_PI)*G
                   *(FR0 + FR1*abs(3.6*V)/100.0)+ FR4*((3.6*V/100.0)**2); 
        
    -- aerodynamic drag resistance  (V and VW in m/s)     
    F_DRAG    == 0.5*RHO*AF*CD*(V+Q_in_VW)**2;
       
    -- grading resistance       
    F_GRAD    == VehicleMass*SIN(Q_in_ALPHA/180.0*MATH_PI)*G;

   -- car dynamic  
    F_CAR     == 0.0; -- DELTA*M*V'LTF((0.0, 1.0),(1.0,TAU));

   -- check tire ground adhesion for front-wheel-driven vehicle
   F_MAX     == MUE * VehicleMass*G*COS(Q_in_ALPHA/180.0*MATH_PI) *
                       (L-LA+FR0*(1.0+V/100.0*3.6)*(HG-RD))/L/(1.0+MUE*HG/L);

   -- determine external force 
   F_EXT == F_CAR + F_GRAD + (F_DRAG + F_ROLLING)*SIGN(V)*F_SMOOTH(V, EPS);

  ELSE

   -- rolling resistance
   -- normal force times rolling resistance coefficient
   -- (note that V in given in m/s)
   F_ROLLING == VehicleMass*COS(Q_in_ALPHA/180.0*MATH_PI)*G
                  *(FR0 + FR1*abs(3.6*V)/100.0)+ FR4*((3.6*V/100.0)**2); 
        
   -- aerodynamic drag resistance  (V and VW in m/s)     
   F_DRAG    == 0.5*RHO*AF*CD*(V+Q_in_VW)**2;
       
   -- grading resistance       
   F_GRAD    == VehicleMass*SIN(Q_in_ALPHA/180.0*MATH_PI)*G;

   -- car dynamic  
   F_CAR     == 0.0 ;--DELTA*M*V'LTF((0.0, 1.0),(1.0,TAU));

   -- check tire ground adhesion for front-wheel-driven vehicle
   F_MAX     == MUE * VehicleMass*G*COS(Q_in_ALPHA/180.0*MATH_PI) *
                       (L-LA+FR0*(1.0+V/100.0*3.6)*(HG-RD))/L/(1.0+MUE*HG/L);

   -- determine external force 
   F_EXT == F_CAR + F_GRAD + (F_DRAG + F_ROLLING)*SIGN(V)*F_SMOOTH(V, EPS);

  END USE;

  assert F_MAX'ABOVE(F_EXT) or (F_MAX >= F_EXT)
       report "WARNING: Tire-Ground adhesion condition is violated."
       severity WARNING;
            
  Fmass == VehicleMass * Vmass'dot ;
  Q_out_Speed == Vmass ; 

end architecture FRONT_DRIVEN;
