----------------------------------------------------------------------
-- 10. Vehicle model :
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE ;
    use IEEE.MECHANICAL_SYSTEMS.all;
    use IEEE.FUNDAMENTAL_CONSTANTS.all;
    use IEEE.MATH_REAL.all;
----------------------------------------------------------------------
entity VEHICLE is
  generic (
    FR0   : REAL :=    0.01;  -- rolling resistance coefficient
    FR1   : REAL :=    0.0;   -- rolling resistance coefficient of v 
    FR4   : REAL :=    0.0;   -- rolling resistance coefficient of v**4
    MUE   : REAL :=    0.75;  -- frictional coefficient
    RHO   : REAL :=    1.2;   -- density of air [kg/m^3]
    TAU   : REAL :=    0.016;  -- time constant [s]
    VehicleMass : REAL :=  1500.0;  -- VehicleMaa [kg]
    AF    : REAL := 2.0;     -- Frontal projected area [m^2]
    L     : REAL := 2.0;     -- Length of wheel base [m]
    LA    : REAL := 0.5;     -- Gravity center position from front wheel[m]
    RD    : REAL := 0.25;     -- wheel radius [m]
    HG    : REAL := 0.5;     -- Gravity center height [m]
    CD    : REAL := 0.5     -- coefficient of aerodynamic resistance
  );
  port   (
    quantity Q_in_VW    : REAL := 0.0 ;          -- wind [m/s]
    quantity Q_in_ALPHA : REAL := 0.0 ;          -- grading angle [deg}
    quantity Q_out_Speed : out REAL := 0.0 ; --vehicle speed [m/s]
    terminal MTV   : TRANSLATIONAL_VELOCITY
  );             
end entity VEHICLE;
----------------------------------------------------------------------
architecture FRONT_DRIVEN of VEHICLE is
  --------------
  function F_SMOOTH(V, EPS : REAL) return REAL is
    variable RESULT : REAL;
  begin
    if abs V < EPS then
      RESULT := 1.0 - COS(V/EPS*MATH_PI/2.0);
    else
      RESULT := 1.0;
    end if;
    return RESULT;
  end function F_SMOOTH; 
  -------------

  constant EPS : REAL := 1.0;
  constant G   : REAL := PHYS_GRAVITY;  -- gravity constant [m/s^-2]
     
  -- V in [m/s], F in N
  quantity V  across F_EXT through MTV;
  
  -- Vehicle Mass
  quantity Vmass across Fmass through MTV to TRANSLATIONAL_V_REF ;

  quantity F_ROLLING : REAL;
  quantity F_CAR     : REAL;
  quantity F_DRAG    : REAL; 
  quantity F_GRAD    : REAL;
  quantity F_MAX     : REAL;
     
begin

  IF (domain = quiescent_domain) USE
    -- rolling resistance
    -- normal force times rolling resistance coefficient
    -- (note that V in given in m/s)
    F_ROLLING == VehicleMass*COS(Q_in_ALPHA/180.0*MATH_PI)*G
                   *(FR0 + FR1*abs(3.6*V)/100.0)+ FR4*((3.6*V/100.0)**2); 
        
    -- aerodynamic drag resistance  (V and VW in m/s)     
    F_DRAG    == 0.5*RHO*AF*CD*(V+Q_in_VW)**2;
       
    -- grading resistance       
    F_GRAD    == VehicleMass*SIN(Q_in_ALPHA/180.0*MATH_PI)*G;

   -- car dynamic  
    F_CAR     == 0.0; -- DELTA*M*V'LTF((0.0, 1.0),(1.0,TAU));

   -- check tire ground adhesion for front-wheel-driven vehicle
   F_MAX     == MUE * VehicleMass*G*COS(Q_in_ALPHA/180.0*MATH_PI) *
                       (L-LA+FR0*(1.0+V/100.0*3.6)*(HG-RD))/L/(1.0+MUE*HG/L);

   -- determine external force 
   F_EXT == F_CAR + F_GRAD + (F_DRAG + F_ROLLING)*SIGN(V)*F_SMOOTH(V, EPS);

  ELSE

   -- rolling resistance
   -- normal force times rolling resistance coefficient
   -- (note that V in given in m/s)
   F_ROLLING == VehicleMass*COS(Q_in_ALPHA/180.0*MATH_PI)*G
                  *(FR0 + FR1*abs(3.6*V)/100.0)+ FR4*((3.6*V/100.0)**2); 
        
   -- aerodynamic drag resistance  (V and VW in m/s)     
   F_DRAG    == 0.5*RHO*AF*CD*(V+Q_in_VW)**2;
       
   -- grading resistance       
   F_GRAD    == VehicleMass*SIN(Q_in_ALPHA/180.0*MATH_PI)*G;

   -- car dynamic  
   F_CAR     == 0.0 ;--DELTA*M*V'LTF((0.0, 1.0),(1.0,TAU));

   -- check tire ground adhesion for front-wheel-driven vehicle
   F_MAX     == MUE * VehicleMass*G*COS(Q_in_ALPHA/180.0*MATH_PI) *
                       (L-LA+FR0*(1.0+V/100.0*3.6)*(HG-RD))/L/(1.0+MUE*HG/L);

   -- determine external force 
   F_EXT == F_CAR + F_GRAD + (F_DRAG + F_ROLLING)*SIGN(V)*F_SMOOTH(V, EPS);

  END USE;

  assert F_MAX'ABOVE(F_EXT) or (F_MAX >= F_EXT)
       report "WARNING: Tire-Ground adhesion condition is violated."
       severity WARNING;
            
  Fmass == VehicleMass * Vmass'dot ;
  Q_out_Speed == Vmass ; 

end architecture FRONT_DRIVEN;
