----------------------------------------------------------------------
-- PMSM 3 Phase Motor : 
--
-- Copyright (C) 2017 JSAE
----------------------------------------------------------------------
library IEEE;
  use IEEE.MATH_REAL.all;
  use IEEE.mechanical_systems.all;
  use IEEE.electrical_systems.all;
----------------------------------------------------------------------
entity motor_pmsm_3ph is
  generic (
    rs : resistance := 0.1; -- Resistance of each stator winding [Ohm]
    L : inductance := 1.0e-3; -- Inductance of each stator winding [Henry]
    kt : real := 1.0; -- Torque constant [N*m/Amp]
    np : real := 2.0; -- Number of poles [No Units]
    d : real := 0.0; -- Viscous drag torque [N*m/(rad/sec)]
    j : moment_inertia := 0.0 -- Moment of inertia [kg*meter**2]
  );
  port (
    terminal sa, sb, sc, sn : electrical;
    terminal shaft, housing : rotational_velocity
  );
end entity motor_pmsm_3ph;
----------------------------------------------------------------------
architecture default of motor_pmsm_3ph is
  quantity vas across ias through sa to sn;
  quantity vbs across ibs through sb to sn;
  quantity vcs across ics through sc to sn;
  quantity vsn across isn through sn to electrical_ref;
  quantity lambda_as, lambda_bs, lambda_cs : magnetic_flux;
  quantity w across torq through shaft to housing;
  quantity te : torque;
  quantity theta_r : angle;
  quantity theta_r_np : angle;
  constant pi2_3 : real := 2.0*math_pi/3.0;
  constant np_2 : real := np/2.0;
  constant lambda_max : magnetic_flux := kt/(1.5*np_2);
  constant r_sn : resistance := 1.0e6;

begin 
--- Flux relationships
  lambda_as == L*(ias - 0.5*ibs - 0.5*ics) + lambda_max*cos(theta_r_np);
  lambda_bs == L*(ibs - 0.5*ias - 0.5*ics) + lambda_max*cos(theta_r_np - pi2_3);
  lambda_cs == L*(ics - 0.5*ibs - 0.5*ias) + lambda_max*cos(theta_r_np + pi2_3);

--- Voltage relationships
  vas == rs*ias + lambda_as'dot;
  vbs == rs*ibs + lambda_bs'dot;
  vcs == rs*ics + lambda_cs'dot;
  vsn == r_sn * isn;

--- Torque and mechanical relationships
  te == -1.0*lambda_max*np_2*(ias*sin(theta_r_np) + ibs*sin(theta_r_np - pi2_3) + ics*sin(theta_r_np + pi2_3));
  torq == -1.0*te + d*w + j*w'dot;
  theta_r == w'integ; --- Assumes initial angle = 0.0 at time 0.
  theta_r_np == theta_r*(np_2);
end architecture default;